package javax.swing.plaf.synth;

import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.ViewportUI;

public class SynthViewportUI extends ViewportUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    
    public SynthViewportUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthViewportUI();
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        this.installDefaults(c);
        this.installListeners(c);
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        this.uninstallListeners(c);
        this.uninstallDefaults(c);
    }
    
    protected void installDefaults(JComponent component) {
        this.updateStyle(component);
    }
    
    private void updateStyle(JComponent component) {
        SynthContext context = this.getContext(component, 1);
        SynthStyle style = SynthLookAndFeel.getStyle(context.getComponent(), context.getRegion());
        SynthStyle style2 = context.getStyle();
        if (style != style2) {
            if (style2 != null) {
                style2.uninstallDefaults(context);
            }
            context.setStyle(style);
            style.installDefaults(context);
        }
        this.style = style;
        context.dispose();
    }
    
    protected void installListeners(JComponent component) {
        component.addPropertyChangeListener(this);
    }
    
    protected void uninstallListeners(JComponent component) {
        component.removePropertyChangeListener(this);
    }
    
    protected void uninstallDefaults(JComponent component) {
        SynthContext context = this.getContext(component, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, SynthLookAndFeel.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private Region getRegion(JComponent component) {
        return SynthLookAndFeel.getRegion(component);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintViewportBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JComponent)propertyChangeEvent.getSource());
        }
    }
}
